<?PHP

/*$mdocInfo
 Author: Hauke Goos-Habermann (HHabermann@pc-kiel.de)
 Description: Debian specific management functions.
$*/


include('clientConfigCommon.php');



define('DEBIAN8DESKTOP_MATE_FULL', 4001);
define('DEBIAN8DESKTOP_MATE_MINIMAL', 4002);
define('DEBIAN8DESKTOP_TRINITY_MINIMAL', 4003);
define('DEBIAN8DESKTOP_GNOME_FULL', 4004);
define('DEBIAN8DESKTOP_XFCE_FULL', 4005);
define('DEBIAN8DESKTOP_KDE_FULL', 4006);
define('DEBIAN8DESKTOP_CINNAMON_FULL', 4007);
define('DEBIAN8DESKTOP_LXDE_FULL', 4008);

define('DEBIAN10DESKTOP_LXQT', 4009);


// #gnome-desktop
// #xfce-desktop
// #kde-desktop
// #cinnamon-desktop
// #mate-desktop
// #lxde-desktop
// lxqt-desktop



/**
**name DEBIAN_desktopInstall($desktop)
**description Installs a Debian desktop.
**parameter desktop: Desktop constant.
**/
function DEBIAN_desktopInstall($desktop)
{
	$lang = getClientLanguage();

	$lV = I18N_getLangVars($lang);

	$session = false;

	include("/m23/inc/i18n/".I18N_m23instLanguage($lang)."/m23inst.php");

	$installDesktopLanguagePackage = true;

	switch($desktop)
	{
		case DEBIAN10DESKTOP_LXQT:
			$desktopPackages = 'task-lxqt-desktop task-desktop xfwm4 xfwm4-theme-breeze sddm';
			$dialogHeader = $I18N_installingLXQT;
			CLCFG_blacklistPackageInstallation('connman');
			$session = 'lxqt.desktop';
			$checkPackage = 'task-lxqt-desktop';
		break;

		case DEBIAN8DESKTOP_MATE_FULL:
			$checkPackage = $desktopPackages = 'task-mate-desktop';
			$dialogHeader = $I18N_installingMate;
		break;

		case DEBIAN8DESKTOP_MATE_MINIMAL:
			$desktopPackages = 'mate-applets mate-applets-common mate-backgrounds mate-control-center mate-control-center-common mate-desktop mate-desktop-common mate-desktop-environment mate-desktop-environment-core mate-icon-theme mate-icon-theme-faenza mate-media mate-media-common mate-media-pulse mate-menus mate-notification-daemon mate-panel mate-panel-common mate-polkit:amd64 mate-polkit-common mate-power-manager mate-power-manager-common mate-screensaver mate-screensaver-common mate-session-manager mate-settings-daemon mate-settings-daemon-common mate-settings-daemon-pulse mate-system-monitor mate-system-monitor-common mate-terminal mate-terminal-common mate-themes mate-utils mate-utils-common';
			$dialogHeader = $I18N_installingMate;
			$checkPackage = 'mate-desktop';
		break;

		case DEBIAN8DESKTOP_TRINITY_MINIMAL:
			$desktopPackages = 'desktop-base-trinity kate-trinity kdesktop-trinity kicker-trinity konsole-trinity kpersonalizer-trinity ksmserver-trinity ksplash-trinity tdebase-runtime-data-common-trinity tdebase-trinity-bin tdm-trinity twin-trinity';
			$dialogHeader = $I18N_installing_trinity;
			$checkPackage = 'desktop-base-trinity';
		break;

		case DEBIAN8DESKTOP_GNOME_FULL:
			$checkPackage = $desktopPackages = 'task-gnome-desktop';
			$dialogHeader = $I18N_installing_gnome;
		break;

		case DEBIAN8DESKTOP_XFCE_FULL:
			$checkPackage = $desktopPackages = 'task-xfce-desktop';
			$dialogHeader = $I18N_installing_xfce;
		break;

		case DEBIAN8DESKTOP_KDE_FULL:
			$desktopPackages = "task-kde-desktop kde-l10n-$lV[packagelang]";
			$dialogHeader = $I18N_installing_kde;
			$checkPackage = 'task-kde-desktop';
		break;

		case DEBIAN8DESKTOP_CINNAMON_FULL:
			$desktopPackages = 'task-cinnamon-desktop dbus-x11';
			$dialogHeader = $I18N_installingCinnamon;
			$session = 'cinnamon.desktop';
			$checkPackage = 'task-cinnamon-desktop';
		break;

		case DEBIAN8DESKTOP_LXDE_FULL:
			$checkPackage = $desktopPackages = 'task-lxde-desktop';
			CLCFG_blacklistPackageInstallation('connman');
			$dialogHeader = $I18N_installingLXDE;
		break;
	}

	if ($installDesktopLanguagePackage)
		CLCFG_installDesktopLanguagePackage($lang, true, true);

	CLCFG_dialogInfoBox($I18N_client_installation, $I18N_client_status, $dialogHeader);
//m23customPatchBegin type=change id=DEBIAN_desktopInstallDisplayManager
	CLCFG_setDebConfDirect('lightdm shared/default-x-display-manager select lightdm');

	// Install the chosen desktop
	CLCFG_aptGet("install", "$desktopPackages");

	// Check, if the needed package is installed
	CLCFG_dpkgPackageInstallationCheck($checkPackage, true);

	// Install lightdm
	CLCFG_aptGet("install", "acpi-support lightdm");
//m23customPatchEnd id=DEBIAN_desktopInstallDisplayManager
	/* =====> */ MSR_statusBarIncCommand(100);

	if ($session !== false)
		CLCFG_installLightDM($session);

// 	echo("\ndpkg-reconfigure m23-skel\n");
}





/**
**name CLCFG_listDebianReleasesGeneric($distr)
**description Generates an array of the different releases (e.g. sarge, sid, woody, potato) of a distribution.
**parameter selName: the name of the selection
**parameter first: the release to show first
**parameter distr: distribution directory
**returns: Array with release names.
**/
function CLCFG_getDebianReleasesGeneric($distr)
{
	$nr = 0;
	$DIR = opendir ("/m23/data+scripts/distr/$distr/debootstrap/scripts/");

	while (false !== ($file = readdir ($DIR)))
	{
		if (strpos($file,".") === false)
			$out[$nr++] = $file;
	}
	
	sort($out);

	closedir($DIR);
	return($out);
};





/**
**name CLCFG_listDebianReleasesGeneric($selName,$first,$distr)
**description generates a selection of the different releases (e.g. sarge, sid, woody, potato) of a distribution.
**parameter selName: the name of the selection
**parameter first: the release to show first
**parameter distr: distribution directory
**returns: Selection with release names.
**/
function CLCFG_listDebianReleasesGeneric($selName,$first,$distr)
{
	return(HTML_listSelection($selName,CLCFG_getDebianReleasesGeneric($distr),$first));
};





/**
**name CLCFG_listDebianReleases($selName,$first)
**description generates a selection of the different Debian releases (sarge, sid, woody, potato)
**parameter selName: the name of the selection
**parameter first: the release to show first
**/
function CLCFG_listDebianReleases($selName,$first)
{
	return(CLCFG_listDebianReleasesGeneric($selName,$first,"debian"));
}
$CLCFG_listReleases = 'CLCFG_listDebianReleases';





/**
**name CLCFG_addDistributionSpecificOptions( $options)
**description adds distribution specific settings to an option array and returns the new array
**parameter $options: the options array with some values
**/
function CLCFG_addDistributionSpecificOptions($options)
{
	$options['kernel']=(isset($_POST['SEL_kernel']) ? $_POST['SEL_kernel'] : NULL);
	$options['disableSSLCertCheck'] = (HTML_getElementValue('CB_disableSSLCertCheck', '', false, true) ? 1 : 0);
	$options['disableSudoRootLogin'] = (HTML_getElementValue('CB_disableSudoRootLogin', '', false, true) ? 1 : 0);
	$options['installX2goserver'] = (HTML_getElementValue('CB_installX2goserver', '', false, true) ? 1 : 0);
	return($options);
};





/**
**name CLCFG_showDistributionSpecificOptions($options, $distr = "debian")
**description shows distribution specific options and returns false, if there was an error
**parameter options: options array
**parameter distr: The name the distribution to use.
**/
function CLCFG_showDistributionSpecificOptions($options, $distr = "debian")
{
	include_once("/m23/inc/distr/debian/packages.php");
	include("/m23/inc/i18n/".$GLOBALS["m23_language"]."/m23base.php");

	$options = CLIENT_getSetOption(NULL,"kernel",$options);

	$kernelList = PKG_getKernels($distr, $_POST['SEL_sourcename'], $options['arch']);
	$kernel = HTML_storableSelection("SEL_kernel", "kernel", $kernelList, SELTYPE_selection, true, isset($options['kernel']) ? $options['kernel'] : NULL, $options['kernel']);
	$options = CLIENT_getSetOption($kernel,"kernel",$options);

	$disableSSLCertCheck = HTML_storableCheckBox('CB_disableSSLCertCheck', '', 'disableSSLCertCheck', isset($options['disableSSLCertCheck']) && ($options['disableSSLCertCheck'] == 1));
	$options = CLIENT_getSetOption(($disableSSLCertCheck ? 1 : 0), 'disableSSLCertCheck', $options);

	$installX2goserver = HTML_storableCheckBox('CB_installX2goserver', '', 'installX2goserver', isset($options['installX2goserver']) && ($options['installX2goserver'] == 1));
	$options = CLIENT_getSetOption(($installX2goserver ? 1 : 0), 'installX2goserver', $options);
	
	
	/***********
	CAUTION: Must be set in CLCFG_addDistributionSpecificOptions too
	***********/

	echo("
	<tr>
		<td>Kernel</td>
		<td colspan=\"2\">".SEL_kernel."</td>
	</tr>
	<tr>
		<td>$I18N_disableSSLCertCheck</td>
		<td colspan=\"2\">".CB_disableSSLCertCheck."</td>
	</tr>
	<tr>
		<td>$I18N_installX2goserver</td>
		<td colspan=\"2\">".CB_installX2goserver."</td>
	</tr>
	");

	return($options);
}
$CLCFG_showDistributionSpecificOptions='CLCFG_showDistributionSpecificOptions';





/**
**name CLIENT_enableLDAP($clientOptions)
**description enables LDAP login for a client.
**parameter clientOptions: the client's options array
**/
function CLCFG_enableLDAPDebian($clientOptions)
{
	//exit the function if LDAP shouldn't used
	if ($clientOptions['ldaptype']=="none")
		return;

	if (HELPER_isExecutedOnUCS())
	{
		UCS_enableClientLDAP();
		return(false);
	}

	// Exit the function if no LDAP server is given
	if (!isset($clientOptions['ldapserver']))
		return(false);

	$server=LDAP_loadServer($clientOptions['ldapserver']);

	$LDAPhost=$server['host'];

	// If the client option "ldapbaseprefix" is set, put it before the baseDN
	if (isset($clientOptions['ldapbaseprefix']))
		$baseDN = $clientOptions['ldapbaseprefix'].$server['base'];
	else
		$baseDN = $server['base'];

	//exit the function if LDAP host or base DN is empty
	if (empty($LDAPhost) || empty($baseDN))
		return;

	$LDAPProtocol = CLIENT_getLDAPProtocol($clientOptions);

		CLCFG_setDebConfDirect("libnss-ldap libnss-ldap/binddn string cn=proxyuser,$baseDN
libnss-ldap libnss-ldap/bindpw password
libnss-ldap libnss-ldap/confperm boolean true
libnss-ldap libnss-ldap/dblogin boolean false
libnss-ldap libnss-ldap/dbrootlogin boolean false
libnss-ldap libnss-ldap/nsswitch note
libnss-ldap libnss-ldap/override boolean true
libnss-ldap libnss-ldap/rootbinddn string cn=manager,$baseDN
libnss-ldap libnss-ldap/rootbindpw password
libnss-ldap shared/ldapns/base-dn string $baseDN
libnss-ldap shared/ldapns/ldap-server string $LDAPhost
libnss-ldap shared/ldapns/ldap_version select 3
libpam-runtime libpam-runtime/override boolean false
libpam-runtime libpam-runtime/profiles multiselect unix, ldap, systemd, mkhomedir
portmap portmap/loopback boolean false
nslcd nslcd/ldap-bindpw password
nslcd nslcd/ldap-sasl-secprops string
nslcd nslcd/ldap-base string $baseDN
nslcd nslcd/ldap-starttls boolean true
nslcd nslcd/ldap-cacertfile string /etc/ssl/certs/ca-certificates.crt
nslcd nslcd/ldap-sasl-krb5-ccname string /var/run/nslcd/nslcd.tkt
nslcd nslcd/ldap-binddn string
nslcd nslcd/ldap-uris string $LDAPhost
nslcd nslcd/ldap-reqcert select never
nslcd nslcd/ldap-sasl-authzid string
nslcd nslcd/ldap-auth-type select SASL
nslcd nslcd/ldap-sasl-authcid string
nslcd nslcd/ldap-sasl-mech select auto
nslcd nslcd/ldap-sasl-realm string");


	//Thanks to the howto from: http://www.debuntu.org/ldap-server-and-linux-ldap-clients-p2

	CLCFG_aptGet('install','libnss-ldap libpam-ldap nscd ldap-utils libnss-db nss-updatedb');
	CLCFG_aptGet('install','nslcd');
	
	// DebianVersionSpecific
	if (('bullseye' != $clientOptions['release']) || ('bookworm' != $clientOptions['release']))
	{
		echo("

	echo \"host $LDAPhost
base $baseDN
rootbinddn cn=admin,$baseDN\" > /etc/libnss-ldap.conf

	cat /etc/libnss-ldap.conf > /etc/pam_ldap.conf

pam configuration files need to be modfied a bit like:

	echo \"account sufficient pam_ldap.so
account required pam_unix.so
session required pam_mkhomedir.so umask=0022 skel=/etc/skel/ silent\" > /etc/pam.d/common-account

	echo \"auth sufficient pam_ldap.so
auth required pam_unix.so nullok_secure use_first_pass\" > /etc/pam.d/common-auth

	echo \"password sufficient pam_ldap.so
password required pam_unix.so nullok obscure min=4 $server[passwordHashAlgorithm]\" > /etc/pam.d/common-password

	echo \"session sufficient pam_ldap.so
session required pam_unix.so
session optional pam_foreground.so\" > /etc/pam.d/common-session


	echo \"BASE dc=$baseDN
URI $LDAPProtocol://$LDAPhost\" > /etc/ldap/ldap.conf
");
	}

// Add the local groups to the LDAP user
echo('
echo "*; *; *; Al0000-2400;'.DISTR_getDebianUserGroups(',').'" >> /etc/security/group.conf
echo "auth required pam_group.so use_first_pass" >> /etc/pam.d/common-auth
');

	CLCFG_patchNsswitchForLDAP();

echo('
pam-auth-update --enable mkhomedir --force
');

	// DebianVersionSpecific
// 	if ('bullseye' == $clientOptions['release'])
// 		CLCFG_setDebConfDirect("libpam-runtime libpam-runtime/profiles multiselect unix, ldap, systemd, mkhomedir");

	echo("
/usr/sbin/nss_updatedb ldap
dpkg-reconfigure libpam-runtime
	");
}
?>